@@ -1,5 +1,5 @@
-/* A simple standalone XML-RPC server written in C as an example of use of
-   the Xmlrpc-c libraries.
+/* A simple standalone XML-RPC server program written in C as an example of
+   use of the Xmlrpc-c libraries.
 
    This example expects an already bound socket on Standard Input, ready to
    be listened on for client connections.  Also see xmlrpc_sample_add_server,
@@ -7,11 +7,17 @@
    creates the socket itself.  Also see xmlrpc_inetd_server.c, which is
    the same thing except you give it a socket which is already connected
    to a client.
- */
+
+   Example:
+
+   $ socketexec -local_port=8080 ./xmlrpc_socket_server
+*/
+
+#define WIN32_LEAN_AND_MEAN  /* required by xmlrpc-c/server_abyss.h */
 
 #include <stdlib.h>
 #include <stdio.h>
-#ifdef WIN32
+#ifdef _WIN32
 #  include <windows.h>
 #else
 #  include <unistd.h>
@@ -23,7 +29,7 @@
 
 #include "config.h"  /* information about this build environment */
 
-#ifdef WIN32
+#ifdef _WIN32
   #define SLEEP(seconds) SleepEx(seconds * 1000, 1);
 #else
   #define SLEEP(seconds) sleep(seconds);
@@ -33,8 +39,8 @@
 static xmlrpc_value *
 sample_add(xmlrpc_env *   const envP,
            xmlrpc_value * const paramArrayP,
-           void *         const serverInfo ATTR_UNUSED,
-           void *         const channelInfo ATTR_UNUSED) {
+           void *         const serverInfo,
+           void *         const channelInfo) {
 
     xmlrpc_int32 x, y, z;
 
