@@ -461,6 +461,13 @@ char *
 RequestHeaderValue(TSession *   const sessionP,
                    const char * const name);
 
+XMLRPC_ABYSS_EXPORTED
+abyss_bool
+RequestAuth(TSession *   const sessionP,
+            const char * const credential,
+            const char * const user,
+            const char * const pass);
+
 XMLRPC_ABYSS_EXPORTED
 abyss_bool
 ResponseAddField(TSession *   const sessionP,
