@@ -18,17 +18,31 @@
 #include "bool.h"
 #include "int.h"
 
-#include <xmlrpc-c/base.h>
+#include <xmlrpc-c/c_util.h>
 #include <xmlrpc-c/util_int.h>
+#include <xmlrpc-c/base.h>
 
 #ifdef __cplusplus
 extern "C" {
 #endif
 
+/*
+  XMLRPC_LIB_EXPORTED marks a symbol in this file that is exported from
+  libxmlrpc.
+
+  XMLRPC_BUILDING_LIB says this compilation is part of libxmlrpc, as
+  opposed to something that _uses_ libxmlrpc.
+*/
+#ifdef XMLRPC_BUILDING_LIB
+#define XMLRPC_LIBINT_EXPORTED XMLRPC_DLLEXPORT
+#else
+#define XMLRPC_LIBINT_EXPORTED
+#endif
 
 struct _xmlrpc_value {
     xmlrpc_type _type;
-    int _refcount;
+    struct lock * lockP;
+    unsigned int refcount;
 
     /* Certain data types store their data directly in the xmlrpc_value. */
     union {
@@ -36,8 +50,13 @@ struct _xmlrpc_value {
         xmlrpc_int64 i8;
         xmlrpc_bool b;
         double d;
-        /* time_t t */
-        void * c_ptr;
+        xmlrpc_datetime dt;
+           /* NOTE: may be invalid! e.g. February 30 */
+        struct {
+            void *              objectP;
+            xmlrpc_cptr_dtor_fn dtor;   // NULL if none
+            void *              dtorContext;
+        } cptr;
     } _value;
     
     /* Other data types use a memory block.
@@ -57,10 +76,6 @@ struct _xmlrpc_value {
        non-XML characters, we have to stretch the definition of XML).
 
        For base64, this is bytes of the byte string, directly.
-
-       For datetime, this is in the same format as the contents of
-       a <dateTime.iso8601> XML element.  That really ought to be changed
-       to time_t some day.
     */
     xmlrpc_mem_block _block;
 
@@ -77,6 +92,15 @@ struct _xmlrpc_value {
            This member is always NULL on a system that does not have
            Unicode wchar functions.
         */
+    void * _cache;
+        /* This is a hack to support the old style memory management in which
+           one gets a pointer into memory that belongs to the xmlrpc_value
+           object; i.e. the caller of xmlrpc_read_datetime_str_old() doesn't
+           get memory that he is responsible for freeing.
+
+           This is essentially a cached value of the result of a
+           xmlrpc_read_datetime_str_old().  NULL means nothing cached.
+        */
 };
 
 #define XMLRPC_ASSERT_VALUE_OK(val) \
@@ -97,24 +121,34 @@ typedef struct {
 } _struct_member;
 
 
+XMLRPC_LIBINT_EXPORTED
 void
 xmlrpc_createXmlrpcValue(xmlrpc_env *    const envP,
                          xmlrpc_value ** const valPP);
 
+XMLRPC_LIBINT_EXPORTED
 const char *
 xmlrpc_typeName(xmlrpc_type const type);
 
+XMLRPC_LIBINT_EXPORTED
 void
 xmlrpc_traceXml(const char * const label, 
                 const char * const xml,
-                unsigned int const xmlLength);
+                size_t       const xmlLength);
 
+XMLRPC_LIBINT_EXPORTED
 void
 xmlrpc_destroyString(xmlrpc_value * const stringP);
 
+XMLRPC_LIBINT_EXPORTED
+void
+xmlrpc_destroyDatetime(xmlrpc_value * const datetimeP);
+
+XMLRPC_LIBINT_EXPORTED
 void
 xmlrpc_destroyStruct(xmlrpc_value * const structP);
 
+XMLRPC_LIBINT_EXPORTED
 void
 xmlrpc_destroyArrayContents(xmlrpc_value * const arrayP);
 
@@ -130,16 +164,19 @@ xmlrpc_destroyArrayContents(xmlrpc_value * const arrayP);
    new memory for them.
 -----------------------------------------------------------------------------*/
 
+XMLRPC_LIBINT_EXPORTED
 void
 xmlrpc_read_datetime_str_old(xmlrpc_env *         const envP,
                              const xmlrpc_value * const valueP,
                              const char **        const stringValueP);
 
+XMLRPC_LIBINT_EXPORTED
 void
 xmlrpc_read_string_old(xmlrpc_env *         const envP,
                        const xmlrpc_value * const valueP,
                        const char **        const stringValueP);
 
+XMLRPC_LIBINT_EXPORTED
 void
 xmlrpc_read_string_lp_old(xmlrpc_env *         const envP,
                           const xmlrpc_value * const valueP,
@@ -147,11 +184,13 @@ xmlrpc_read_string_lp_old(xmlrpc_env *         const envP,
                           const char **        const stringValueP);
 
 #if XMLRPC_HAVE_WCHAR
+XMLRPC_LIBINT_EXPORTED
 void
 xmlrpc_read_string_w_old(xmlrpc_env *     const envP,
                          xmlrpc_value *   const valueP,
                          const wchar_t ** const stringValueP);
 
+XMLRPC_LIBINT_EXPORTED
 void
 xmlrpc_read_string_w_lp_old(xmlrpc_env *     const envP,
                             xmlrpc_value *   const valueP,
@@ -159,6 +198,7 @@ xmlrpc_read_string_w_lp_old(xmlrpc_env *     const envP,
                             const wchar_t ** const stringValueP);
 #endif
 
+XMLRPC_LIBINT_EXPORTED
 void
 xmlrpc_read_base64_old(xmlrpc_env *           const envP,
                        const xmlrpc_value *   const valueP,
