@@ -7,14 +7,16 @@
 #include <ctype.h>
 #include <assert.h>
 
+#include "bool.h"
 #include "mallocvar.h"
 #include "xmlrpc-c/util_int.h"
 #include "xmlrpc-c/string_int.h"
 #include "xmlrpc-c/sleep_int.h"
 #include "xmlrpc-c/abyss.h"
-#include "socket.h"
+#include "channel.h"
 #include "server.h"
 #include "thread.h"
+#include "file.h"
 
 #include "conn.h"
 
@@ -29,6 +31,8 @@ connJob(void * const userHandle) {
 /*----------------------------------------------------------------------------
    This is the root function for a thread that processes a connection
    (performs HTTP transactions).
+
+   We never return.  We ultimately exit the thread.
 -----------------------------------------------------------------------------*/
     TConn * const connectionP = userHandle;
 
@@ -42,11 +46,22 @@ connJob(void * const userHandle) {
            after we exit.
         */
 
-    ThreadExit(0);
+
+    /* Note that ThreadExit() runs a cleanup function, which in our
+       case is connDone().
+    */
+    ThreadExit(connectionP->threadP, 0);
 }
 
 
 
+/* This is the maximum amount of stack that 'connJob' itself uses --
+   does not count what user's connection job function uses.
+*/
+#define CONNJOB_STACK 1024
+
+
+
 static void
 connDone(TConn * const connectionP) {
 
@@ -76,7 +91,8 @@ threadDone(void * const userHandle) {
 static void
 makeThread(TConn *             const connectionP,
            enum abyss_foreback const foregroundBackground,
-           abyss_bool          const useSigchld,
+           bool                const useSigchld,
+           size_t              const jobStackSize,
            const char **       const errorP) {
            
     switch (foregroundBackground) {
@@ -89,6 +105,7 @@ makeThread(TConn *             const connectionP,
         connectionP->hasOwnThread = TRUE;
         ThreadCreate(&connectionP->threadP, connectionP,
                      &connJob, &threadDone, useSigchld,
+                     CONNJOB_STACK + jobStackSize,
                      &error);
         if (error) {
             xmlrpc_asprintf(errorP, "Unable to create thread to "
@@ -105,18 +122,20 @@ makeThread(TConn *             const connectionP,
 void
 ConnCreate(TConn **            const connectionPP,
            TServer *           const serverP,
-           TSocket *           const connectedSocketP,
+           TChannel *          const channelP,
+           void *              const channelInfoP,
            TThreadProc *       const job,
+           size_t              const jobStackSize,
            TThreadDoneFn *     const done,
            enum abyss_foreback const foregroundBackground,
-           abyss_bool          const useSigchld,
+           bool                const useSigchld,
            const char **       const errorP) {
 /*----------------------------------------------------------------------------
    Create an HTTP connection.
 
    A connection carries one or more HTTP transactions (request/response).
 
-   'connectedSocketP' transports the requests and responses.
+   *channelP transports the requests and responses.
 
    The connection handles those HTTP requests.
 
@@ -133,6 +152,8 @@ ConnCreate(TConn **            const connectionPP,
 
    Some time after the HTTP transactions are all done, 'done' gets
    called in some context.
+
+   'channelInfoP' == NULL means no channel info supplied.
 -----------------------------------------------------------------------------*/
     TConn * connectionP;
 
@@ -142,34 +163,28 @@ ConnCreate(TConn **            const connectionPP,
         xmlrpc_asprintf(errorP, "Unable to allocate memory for a connection "
                         "descriptor.");
     else {
-        abyss_bool success;
-        uint16_t peerPortNumber;
+        connectionP->server       = serverP;
+        connectionP->channelP     = channelP;
+        connectionP->channelInfoP = channelInfoP;
+        connectionP->buffer.b[0]  = '\0';
+        connectionP->buffersize   = 0;
+        connectionP->bufferpos    = 0;
+        connectionP->finished     = FALSE;
+        connectionP->job          = job;
+        connectionP->done         = done;
+        connectionP->inbytes      = 0;
+        connectionP->outbytes     = 0;
+        connectionP->trace        = getenv("ABYSS_TRACE_CONN");
 
-        connectionP->server     = serverP;
-        connectionP->socketP    = connectedSocketP;
-        connectionP->buffersize = 0;
-        connectionP->bufferpos  = 0;
-        connectionP->finished   = FALSE;
-        connectionP->job        = job;
-        connectionP->done       = done;
-        connectionP->inbytes    = 0;
-        connectionP->outbytes   = 0;
-        connectionP->trace      = getenv("ABYSS_TRACE_CONN");
-
-        SocketGetPeerName(connectedSocketP,
-                          &connectionP->peerip, &peerPortNumber, &success);
-
-        if (success)
-            makeThread(connectionP, foregroundBackground, useSigchld, errorP);
-        else
-            xmlrpc_asprintf(errorP, "Failed to get peer name from socket.");
+        makeThread(connectionP, foregroundBackground, useSigchld,
+                   jobStackSize, errorP);
     }
     *connectionPP = connectionP;
 }
 
 
 
-abyss_bool
+bool
 ConnProcess(TConn * const connectionP) {
 /*----------------------------------------------------------------------------
    Drive the main processing of a connection -- run the connection's
@@ -181,12 +196,13 @@ ConnProcess(TConn * const connectionP) {
    we return or some time after.  If we fail, we guarantee the "done"
    function will not be called.
 -----------------------------------------------------------------------------*/
-    abyss_bool retval;
+    bool retval;
 
     if (connectionP->hasOwnThread) {
         /* There's a background thread to handle this connection.  Set
            it running.
         */
+        assert(connectionP->threadP);
         retval = ThreadRun(connectionP->threadP);
     } else {
         /* No background thread.  We just handle it here while Caller waits. */
@@ -201,16 +217,18 @@ ConnProcess(TConn * const connectionP) {
 
 void
 ConnWaitAndRelease(TConn * const connectionP) {
-    if (connectionP->hasOwnThread)
+
+    if (connectionP->hasOwnThread) {
+        assert(connectionP->threadP);
         ThreadWaitAndRelease(connectionP->threadP);
-    
+    }
     free(connectionP);
 }
 
 
 
-abyss_bool
-ConnKill(TConn * connectionP) {
+bool
+ConnKill(TConn * const connectionP) {
     connectionP->finished = TRUE;
     return ThreadKill(connectionP->threadP);
 }
@@ -219,65 +237,120 @@ ConnKill(TConn * connectionP) {
 
 void
 ConnReadInit(TConn * const connectionP) {
-    if (connectionP->buffersize>connectionP->bufferpos) {
+
+    if (connectionP->buffersize > connectionP->bufferpos) {
         connectionP->buffersize -= connectionP->bufferpos;
-        memmove(connectionP->buffer,
-                connectionP->buffer+connectionP->bufferpos,
+        memmove(connectionP->buffer.b,
+                connectionP->buffer.b + connectionP->bufferpos,
                 connectionP->buffersize);
         connectionP->bufferpos = 0;
     } else
-        connectionP->buffersize=connectionP->bufferpos = 0;
+        connectionP->buffersize = connectionP->bufferpos = 0;
 
-    connectionP->inbytes=connectionP->outbytes = 0;
+    connectionP->buffer.b[connectionP->buffersize] = '\0';
+
+    connectionP->inbytes = connectionP->outbytes = 0;
 }
 
 
 
 static void
-traceBuffer(const char * const label,
-            const char * const buffer,
-            unsigned int const size) {
-
-    unsigned int nonPrintableCount;
-    unsigned int i;
-    
-    nonPrintableCount = 0;  /* Initial value */
-    
-    for (i = 0; i < size; ++i) {
-        if (!isprint(buffer[i]) && buffer[i] != '\n' && buffer[i] != '\r')
-            ++nonPrintableCount;
-    }
-    if (nonPrintableCount > 0)
-        fprintf(stderr, "%s contains %u nonprintable characters.\n", 
-                label, nonPrintableCount);
-    
-    fprintf(stderr, "%s:\n", label);
-    fprintf(stderr, "%.*s\n", (int)size, buffer);
-}
-
-
-
-static void
-traceSocketRead(TConn *      const connectionP,
-                unsigned int const size) {
+traceReadTimeout(TConn *  const connectionP,
+                 uint32_t const timeout) {
 
     if (connectionP->trace)
-        traceBuffer("READ FROM SOCKET",
-                    connectionP->buffer + connectionP->buffersize, size);
+        fprintf(stderr, "TIMED OUT waiting over %u seconds "
+                "for data from client.\n", timeout);
+}
+
+
+
+static size_t
+nextLineSize(const char * const string,
+             size_t       const startPos,
+             size_t       const stringSize) {
+/*----------------------------------------------------------------------------
+   Return the length of the line that starts at offset 'startPos' in the
+   string 'string', which is 'stringSize' characters long.
+
+   'string' in not NUL-terminated.
+   
+   A line begins at beginning of string or after a newline character and
+   runs through the next newline character or end of string.  The line
+   includes the newline character at the end, if any.
+-----------------------------------------------------------------------------*/
+    size_t i;
+
+    for (i = startPos; i < stringSize && string[i] != '\n'; ++i);
+
+    if (i < stringSize)
+        ++i;  /* Include the newline */
+
+    return i - startPos;
 }
 
 
 
 static void
-traceSocketWrite(TConn *      const connectionP,
-                 const char * const buffer,
-                 unsigned int const size,
-                 abyss_bool   const failed) {
+traceBuffer(const char *          const label,
+            const unsigned char * const buffer,
+            unsigned int          const size) {
 
+    const char * const buffer_t = (const char *)buffer;
+
+    size_t cursor;  /* Index into buffer[] */
+
+    fprintf(stderr, "%s:\n\n", label);
+
+    for (cursor = 0; cursor < size; ) {
+        /* Print one line of buffer */
+
+        size_t const lineSize = nextLineSize(buffer_t, cursor, size);
+        const char * const printableLine =
+            xmlrpc_makePrintable_lp(&buffer_t[cursor], lineSize);
+        
+        fprintf(stderr, "%s\n", printableLine);
+
+        cursor += lineSize;
+
+        xmlrpc_strfree(printableLine);
+    }
+    fprintf(stderr, "\n");
+}
+
+
+
+static void
+traceBufferText(const char * const label,
+                const char * const buffer,
+                unsigned int const size) {
+
+    traceBuffer(label, (const unsigned char *)buffer, size);
+}
+
+
+
+static void
+traceChannelRead(TConn *      const connectionP,
+                 unsigned int const size) {
+
+    if (connectionP->trace)
+        traceBuffer("READ FROM CHANNEL",
+                    connectionP->buffer.b + connectionP->buffersize, size);
+}
+
+
+
+static void
+traceChannelWrite(TConn *      const connectionP,
+                  const char * const buffer,
+                  unsigned int const size,
+                  bool         const failed) {
+    
     if (connectionP->trace) {
         const char * const label =
-            failed ? "FAILED TO WRITE TO SOCKET" : "WROTE TO SOCKET";
-        traceBuffer(label, buffer, size);
+            failed ? "FAILED TO WRITE TO CHANNEL" : "WROTE TO CHANNEL";
+        traceBufferText(label, buffer, size);
     }
 }
 
@@ -291,82 +364,150 @@ bufferSpace(TConn * const connectionP) {
                     
 
 
-abyss_bool
-ConnRead(TConn *  const connectionP,
-         uint32_t const timeout) {
+static void
+readFromChannel(TConn *       const connectionP,
+                bool *        const eofP,
+                const char ** const errorP) {
 /*----------------------------------------------------------------------------
-   Read some stuff on connection *connectionP from the socket.
+   Read some data from the channel of Connection *connectionP.
 
-   Don't wait more than 'timeout' seconds for data to arrive.  Fail if
-   nothing arrives within that time.
+   Iff there is none available to read, return *eofP == true.
 -----------------------------------------------------------------------------*/
-    time_t const deadline = time(NULL) + timeout;
+    uint32_t bytesRead;
+    bool readError;
 
-    abyss_bool cantGetData;
-    abyss_bool gotData;
+    ChannelRead(connectionP->channelP,
+                connectionP->buffer.b + connectionP->buffersize,
+                bufferSpace(connectionP) - 1,
+                &bytesRead, &readError);
 
-    cantGetData = FALSE;
-    gotData = FALSE;
-    
-    while (!gotData && !cantGetData) {
-        int const timeLeft = deadline - time(NULL);
+    if (readError)
+        xmlrpc_asprintf(errorP, "Error reading from channel");
+    else {
+        *errorP = NULL;
+        if (bytesRead > 0) {
+            *eofP = FALSE;
+            traceChannelRead(connectionP, bytesRead);
+            connectionP->inbytes += bytesRead;
+            connectionP->buffersize += bytesRead;
+            connectionP->buffer.t[connectionP->buffersize] = '\0';
+        } else
+            *eofP = TRUE;
+    }
+}
 
-        if (timeLeft <= 0)
-            cantGetData = TRUE;
+
+
+static void
+dealWithReadTimeout(bool *        const timedOutP,
+                    bool          const timedOut,
+                    uint32_t      const timeout,
+                    const char ** const errorP) {
+
+    if (timedOutP)
+        *timedOutP = timedOut;
+    else {
+        if (timedOut)
+            xmlrpc_asprintf(errorP, "Read from Abyss client "
+                            "connection timed out after %u seconds "
+                            "or was interrupted",
+                            timeout);
+    }
+}
+
+
+
+static void
+dealWithReadEof(bool *        const eofP,
+                bool          const eof,
+                const char ** const errorP) {
+
+    if (eofP)
+        *eofP = eof;
+    else {
+        if (eof)
+            xmlrpc_asprintf(errorP, "Read from Abyss client "
+                            "connection failed because client closed the "
+                            "connection");
+    }
+}
+
+
+
+void
+ConnRead(TConn *       const connectionP,
+         uint32_t      const timeout,
+         bool *        const eofP,
+         bool *        const timedOutP,
+         const char ** const errorP) {
+/*----------------------------------------------------------------------------
+   Read some stuff on connection *connectionP from the channel.  Read it into
+   the connection's buffer.
+
+   Don't wait more than 'timeout' seconds for data to arrive.  If no data has
+   arrived by then and 'timedOutP' is null, fail.  If 'timedOut' is non-null,
+   return as *timedOutP whether 'timeout' seconds passed without any data
+   arriving.
+
+   Also, stop waiting upon any interruption and treat it the same as a
+   timeout.  An interruption is either a signal received (and caught) at
+   an appropriate time or a ChannelInterrupt() call before or during the
+   wait.
+
+   If 'eofP' is non-null, return *eofP == true, without reading anything, iff
+   there will no more data forthcoming on the connection because client has
+   closed the connection.  If 'eofP' is null, fail in that case.
+-----------------------------------------------------------------------------*/
+    uint32_t const timeoutMs = timeout * 1000;
+
+    if (timeoutMs < timeout)
+        /* Arithmetic overflow */
+        xmlrpc_asprintf(errorP, "Timeout value is too large");
+    else {
+        bool const waitForRead  = TRUE;
+        bool const waitForWrite = FALSE;
+
+        bool readyForRead;
+        bool failed;
+            
+        ChannelWait(connectionP->channelP, waitForRead, waitForWrite,
+                    timeoutMs, &readyForRead, NULL, &failed);
+            
+        if (failed)
+            xmlrpc_asprintf(errorP,
+                            "Wait for stuff to arrive from client failed.");
         else {
-            int rc;
-            
-            rc = SocketWait(connectionP->socketP, TRUE, FALSE,
-                            timeLeft * 1000);
-            
-            if (rc != 1)
-                cantGetData = TRUE;
-            else {
-                uint32_t bytesAvail;
-            
-                bytesAvail = SocketAvailableReadBytes(connectionP->socketP);
-                
-                if (bytesAvail <= 0)
-                    cantGetData = TRUE;
-                else {
-                    uint32_t const bytesToRead =
-                        MIN(bytesAvail, bufferSpace(connectionP)-1);
-
-                    uint32_t bytesRead;
-
-                    bytesRead = SocketRead(
-                        connectionP->socketP,
-                        connectionP->buffer + connectionP->buffersize,
-                        bytesToRead);
-                    if (bytesRead > 0) {
-                        traceSocketRead(connectionP, bytesRead);
-                        connectionP->inbytes += bytesRead;
-                        connectionP->buffersize += bytesRead;
-                        connectionP->buffer[connectionP->buffersize] = '\0';
-                        gotData = TRUE;
-                    }
-                }
+            bool eof;
+            if (readyForRead) {
+                readFromChannel(connectionP, &eof, errorP);
+            } else {
+                /* Wait was interrupted, either by our requested timeout,
+                   a (caught) signal, or a ChannelInterrupt().
+                */
+                traceReadTimeout(connectionP, timeout);
+                *errorP = NULL;
+                eof = FALSE;
             }
+            if (!*errorP)
+                dealWithReadTimeout(timedOutP, !readyForRead, timeout, errorP);
+            if (!*errorP)
+                dealWithReadEof(eofP, eof, errorP);
         }
     }
-    if (gotData)
-        return TRUE;
-    else
-        return FALSE;
 }
 
 
             
-abyss_bool
+bool
 ConnWrite(TConn *      const connectionP,
           const void * const buffer,
           uint32_t     const size) {
 
-    abyss_bool failed;
+    bool failed;
 
-    SocketWrite(connectionP->socketP, buffer, size, &failed);
+    ChannelWrite(connectionP->channelP, buffer, size, &failed);
 
-    traceSocketWrite(connectionP, buffer, size, failed);
+    traceChannelWrite(connectionP, buffer, size, failed);
 
     if (!failed)
         connectionP->outbytes += size;
@@ -376,14 +517,14 @@ ConnWrite(TConn *      const connectionP,
 
 
 
-abyss_bool
-ConnWriteFromFile(TConn *  const connectionP,
-                  TFile *  const fileP,
-                  uint64_t const start,
-                  uint64_t const last,
-                  void *   const buffer,
-                  uint32_t const buffersize,
-                  uint32_t const rate) {
+bool
+ConnWriteFromFile(TConn *       const connectionP,
+                  const TFile * const fileP,
+                  uint64_t      const start,
+                  uint64_t      const last,
+                  void *        const buffer,
+                  uint32_t      const buffersize,
+                  uint32_t      const rate) {
 /*----------------------------------------------------------------------------
    Write the contents of the file stream *fileP, from offset 'start'
    up through 'last', to the HTTP connection *connectionP.
@@ -392,9 +533,9 @@ ConnWriteFromFile(TConn *  const connectionP,
 
    Use the 'bufferSize' bytes at 'buffer' as an internal buffer for this.
 -----------------------------------------------------------------------------*/
-    abyss_bool retval;
+    bool retval;
     uint32_t waittime;
-    abyss_bool success;
+    bool success;
     uint32_t readChunkSize;
 
     if (rate > 0) {
@@ -415,10 +556,13 @@ ConnWriteFromFile(TConn *  const connectionP,
         bytesread = 0;  /* initial value */
 
         while (bytesread < totalBytesToRead) {
-            uint64_t const bytesLeft = totalBytesToRead - bytesread;
-            uint64_t const bytesToRead = MIN(readChunkSize, bytesLeft);
+            uint64_t const bytesLeft     = totalBytesToRead - bytesread;
+            uint64_t const bytesToRead64 = MIN(readChunkSize, bytesLeft);
+            uint32_t const bytesToRead   = (uint32_t)bytesToRead64;
+            
+            uint32_t bytesReadThisTime;
 
-            uint64_t bytesReadThisTime;
+            assert(bytesToRead == bytesToRead64); /* readChunkSize is uint32 */
 
             bytesReadThisTime = FileRead(fileP, buffer, bytesToRead);
             bytesread += bytesReadThisTime;
@@ -438,163 +582,6 @@ ConnWriteFromFile(TConn *  const connectionP,
 
 
 
-static void
-processHeaderLine(char *       const start,
-                  const char * const headerStart,
-                  TConn *      const connectionP,                  
-                  time_t       const deadline,
-                  abyss_bool * const gotHeaderP,
-                  char **      const nextP,
-                  abyss_bool * const errorP) {
-/*----------------------------------------------------------------------------
-  If there's enough data in the buffer, process a line of HTTP
-  header.
-
-  It is part of a header that starts at 'headerStart' and has been
-  previously processed up to the line starting at 'start'.  The data
-  in the buffer is terminated by a NUL.
-
-  Return as *nextP the location of the next header line to process
-  (same as 'start' if we didn't find a line to process).
-
-  WE MODIFY THE DATA.
------------------------------------------------------------------------------*/
-    abyss_bool gotHeader;
-    char * lfPos;
-    char * p;
-
-    p = start;
-
-    gotHeader = FALSE;  /* initial assumption */
-
-    lfPos = strchr(p, LF);
-    if (lfPos) {
-        if ((*p != LF) && (*p != CR)) {
-            /* We're looking at a non-empty line */
-            if (*(lfPos+1) == '\0') {
-                /* There's nothing in the buffer after the line, so we
-                   don't know if there's a continuation line coming.
-                   Must read more.
-                */
-                int const timeLeft = deadline - time(NULL);
-                
-                *errorP = !ConnRead(connectionP, timeLeft);
-            }
-            if (!*errorP) {
-                p = lfPos; /* Point to LF */
-                
-                /* If the next line starts with whitespace, it's a
-                   continuation line, so blank out the line
-                   delimiter (LF or CRLF) so as to join the next
-                   line with this one.
-                */
-                if ((*(p+1) == ' ') || (*(p+1) == '\t')) {
-                    if (p > headerStart && *(p-1) == CR)
-                        *(p-1) = ' ';
-                    *p++ = ' ';
-                } else
-                    gotHeader = TRUE;
-            }
-        } else {
-            /* We're looking at an empty line (i.e. what marks the
-               end of the header)
-            */
-            p = lfPos;  /* Point to LF */
-            gotHeader = TRUE;
-        }
-    }
-
-    if (gotHeader) {
-        /* 'p' points to the final LF */
-
-        /* Replace the LF or the CR in CRLF with NUL, so as to terminate
-           the string at 'headerStart' that is the full header.
-        */
-        if (p > headerStart && *(p-1) == CR)
-            *(p-1) = '\0';  /* NUL out CR in CRLF */
-        else
-            *p = '\0';  /* NUL out LF */
-
-        ++p;  /* Point to next line in buffer */
-    }
-    *gotHeaderP = gotHeader;
-    *nextP = p;
-}
-
-
-
-abyss_bool
-ConnReadHeader(TConn * const connectionP,
-               char ** const headerP) {
-/*----------------------------------------------------------------------------
-   Read an HTTP header on connection *connectionP.
-
-   An HTTP header is basically a line, except that if a line starts
-   with white space, it's a continuation of the previous line.  A line
-   is delimited by either LF or CRLF.
-
-   In the course of reading, we read at least one character past the
-   line delimiter at the end of the header; we may read much more.  We
-   leave everything after the header (and its line delimiter) in the
-   internal buffer, with the buffer pointer pointing to it.
-
-   We use stuff already in the internal buffer (perhaps left by a
-   previous call to this subroutine) before reading any more from from
-   the socket.
-
-   Return as *headerP the header value.  This is in the connection's
-   internal buffer.  This contains no line delimiters.
------------------------------------------------------------------------------*/
-    uint32_t const deadline = time(NULL) + connectionP->server->srvP->timeout;
-
-    abyss_bool retval;
-    char * p;
-    char * headerStart;
-    abyss_bool error;
-    abyss_bool gotHeader;
-
-    p = connectionP->buffer + connectionP->bufferpos;
-    headerStart = p;
-
-    gotHeader = FALSE;
-    error = FALSE;
-
-    while (!gotHeader && !error) {
-        int const timeLeft = deadline - time(NULL);
-
-        if (timeLeft <= 0)
-            error = TRUE;
-        else {
-            if (p >= connectionP->buffer + connectionP->buffersize
-                || !strchr(p, LF))
-                /* There is no line yet in the buffer.
-                   Need more data from the socket to chew on
-                */
-                error = !ConnRead(connectionP, timeLeft);
-
-            if (!error) {
-                assert(connectionP->buffer + connectionP->buffersize > p);
-                processHeaderLine(p, headerStart, connectionP, deadline,
-                                  &gotHeader, &p, &error);
-            }
-        }
-    }
-    if (gotHeader) {
-        /* We've consumed this part of the buffer (but be careful --
-           you can't reuse that part of the buffer because the string
-           we're returning is in it!
-        */
-        connectionP->bufferpos += p - headerStart;
-        *headerP = headerStart;
-        retval = TRUE;
-    } else
-        retval = FALSE;
-
-    return retval;
-}
-
-
-
 TServer *
 ConnServer(TConn * const connectionP) {
     return connectionP->server;
@@ -602,6 +589,15 @@ ConnServer(TConn * const connectionP) {
 
 
 
+void
+ConnFormatClientAddr(TConn *       const connectionP,
+                     const char ** const clientAddrPP) {
+
+    ChannelFormatPeerInfo(connectionP->channelP, clientAddrPP);
+}
+
+
+
 /*******************************************************************************
 **
 ** conn.c
