@@ -5,13 +5,16 @@
 
    Copyright information is at end of file
 =============================================================================*/
+
+#define _XOPEN_SOURCE 600  /* Make sure strdup() is in <string.h> */
+
 #include <assert.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 #include <time.h>
 #include <errno.h>
-#ifdef WIN32
+#ifdef _WIN32
   #include <io.h>
 #else
   #include <unistd.h>
@@ -507,17 +510,17 @@ sendBody(TSession *      const sessionP,
     char buffer[4096];
 
     if (sessionP->ranges.size == 0)
-        ConnWriteFromFile(sessionP->conn, fileP, 0, filesize - 1,
+        ConnWriteFromFile(sessionP->connP, fileP, 0, filesize - 1,
                           buffer, sizeof(buffer), 0);
     else if (sessionP->ranges.size == 1)
-        ConnWriteFromFile(sessionP->conn, fileP, start0, end0,
+        ConnWriteFromFile(sessionP->connP, fileP, start0, end0,
                           buffer, sizeof(buffer), 0);
     else {
         uint64_t i;
         for (i = 0; i <= sessionP->ranges.size; ++i) {
-            ConnWrite(sessionP->conn, "--", 2);
-            ConnWrite(sessionP->conn, BOUNDARY, strlen(BOUNDARY));
-            ConnWrite(sessionP->conn, CRLF, 2);
+            ConnWrite(sessionP->connP, "--", 2);
+            ConnWrite(sessionP->connP, BOUNDARY, strlen(BOUNDARY));
+            ConnWrite(sessionP->connP, CRLF, 2);
 
             if (i < sessionP->ranges.size) {
                 uint64_t start;
@@ -534,12 +537,12 @@ sendBody(TSession *      const sessionP,
                     composeEntityHeader(&entityHeader, mediatype,
                                         start, end, filesize);
 
-                    ConnWrite(sessionP->conn,
+                    ConnWrite(sessionP->connP,
                               entityHeader, strlen(entityHeader));
 
                     xmlrpc_strfree(entityHeader);
                     
-                    ConnWriteFromFile(sessionP->conn, fileP, start, end,
+                    ConnWriteFromFile(sessionP->connP, fileP, start, end,
                                       buffer, sizeof(buffer), 0);
                 }
             }
@@ -641,7 +644,7 @@ handleFile(TSession *   const sessionP,
 static void
 convertToNativeFileName(char * const fileName ATTR_UNUSED) {
 
-#ifdef WIN32
+#ifdef _WIN32
     char * p;
     p = &fileName[0];
     while (*p) {
@@ -650,7 +653,7 @@ convertToNativeFileName(char * const fileName ATTR_UNUSED) {
 
         ++p;
     }
-#endif  /* WIN32 */
+#endif  /* _WIN32 */
 }
 
 
