@@ -4,14 +4,17 @@
 #include <sys/types.h>
 
 #include "bool.h"
+#include "xmlrpc-c/lock.h"
 #include "xmlrpc-c/abyss.h"
 
 #include "data.h"
 
 struct TFile;
-struct abyss_mutex;
 
 struct _TServer {
+    bool traceIsActive;
+        /* We should report to Standard Error our internal activities */
+
     bool terminationRequested;
         /* User wants this server to terminate as soon as possible,
            in particular before accepting any more connections and without
@@ -32,7 +35,7 @@ struct _TServer {
     const char * logfilename;
     bool logfileisopen;
     struct TFile * logfileP;
-    struct abyss_mutex * logmutexP;
+    lock * logLockP;
     const char * name;
     bool serverAcceptsConnections;
         /* We listen for and accept TCP connections for HTTP transactions.
@@ -49,6 +52,17 @@ struct _TServer {
         /* Maximum time in seconds the server will wait to read a header
            or a data chunk from the channel.
         */
+    uint32_t maxConn;
+        /* Maximum number of connections the server allows to exist (i.e.
+           HTTP transactions in progress) at once.  Server will not accept
+           a connection if it already has this many.
+        */
+    uint32_t maxConnBacklog;
+        /* Maximum number of connections the server allows the OS to queue
+           waiting for the server to accept it.  The OS accepts this many TCP
+           connections on the server's behalf and holds them waiting for the
+           server to accept them from the OS.
+        */
     TList handlers;
         /* Ordered list of HTTP request handlers.  For each HTTP request,
            Server calls each one in order until one reports that it handled
@@ -81,11 +95,11 @@ struct _TServer {
            of the function itself, not the stack size for the thread
            that runs it.
         */
-#ifndef WIN32
+#ifndef _WIN32
     uid_t uid;
     gid_t gid;
-    struct TFile * pidfileP;
 #endif
+    struct TFile * pidfileP;
 };
 
 #endif
