@@ -23,28 +23,34 @@ SessionRefillBuffer(TSession * const sessionP) {
 
    I.e. read data from the socket.
 -----------------------------------------------------------------------------*/
-    struct _TServer * const srvP = sessionP->conn->server->srvP;
+    struct _TServer * const srvP = sessionP->connP->server->srvP;
     bool failed;
 
     failed = FALSE;  /* initial value */
             
     /* Reset our read buffer & flush data from previous reads. */
-    ConnReadInit(sessionP->conn);
+    ConnReadInit(sessionP->connP);
 
     if (sessionP->continueRequired)
         failed = !HTTPWriteContinue(sessionP);
 
     if (!failed) {
+        const char * readError;
+
         sessionP->continueRequired = FALSE;
 
-        /* Read more network data into our buffer.  If we encounter a
-           timeout, exit immediately.  We're very forgiving about the
-           timeout here.  We allow a full timeout per network read, which
-           would allow somebody to keep a connection alive nearly
-           indefinitely.  But it's hard to do anything intelligent here
-           without very complicated code.
+        /* Read more network data into our buffer.  Fail if we time out before
+           client sends any data or client closes the connection or there's
+           some network error.  We're very forgiving about the timeout here.
+           We allow a full timeout per network read, which would allow
+           somebody to keep a connection alive nearly indefinitely.  But it's
+           hard to do anything intelligent here without very complicated code.
         */
-        failed = !ConnRead(sessionP->conn, srvP->timeout);	
+        ConnRead(sessionP->connP, srvP->timeout, NULL, NULL, &readError);	
+        if (readError) {
+            failed = TRUE;
+            xmlrpc_strfree(readError);
+        }
     }
     return !failed;
 }
@@ -54,7 +60,7 @@ SessionRefillBuffer(TSession * const sessionP) {
 size_t
 SessionReadDataAvail(TSession * const sessionP) {
 
-    return sessionP->conn->buffersize - sessionP->conn->bufferpos;
+    return sessionP->connP->buffersize - sessionP->connP->bufferpos;
 
 }
 
@@ -73,18 +79,18 @@ SessionGetReadData(TSession *    const sessionP,
    We return a pointer to the first byte as *outStartP, and the length in
    bytes as *outLenP.  The memory pointed to belongs to the session.
 -----------------------------------------------------------------------------*/
-    uint32_t const bufferPos = sessionP->conn->bufferpos;
+    uint32_t const bufferPos = sessionP->connP->bufferpos;
 
-    *outStartP = &sessionP->conn->buffer.t[bufferPos];
+    *outStartP = &sessionP->connP->buffer.t[bufferPos];
 
-    assert(bufferPos <= sessionP->conn->buffersize);
+    assert(bufferPos <= sessionP->connP->buffersize);
 
-    *outLenP = MIN(max, sessionP->conn->buffersize - bufferPos);
+    *outLenP = MIN(max, sessionP->connP->buffersize - bufferPos);
 
     /* move pointer past the bytes we are returning */
-    sessionP->conn->bufferpos += *outLenP;
+    sessionP->connP->bufferpos += *outLenP;
 
-    assert(sessionP->conn->bufferpos <= sessionP->conn->buffersize);
+    assert(sessionP->connP->bufferpos <= sessionP->connP->buffersize);
 }
 
 
@@ -102,7 +108,7 @@ void
 SessionGetChannelInfo(TSession * const sessionP,
                       void **    const channelInfoPP) {
     
-    *channelInfoPP = sessionP->conn->channelInfoP;
+    *channelInfoPP = sessionP->connP->channelInfoP;
 }
 
 
@@ -125,25 +131,24 @@ SessionLog(TSession * const sessionP) {
     
     DateToLogString(sessionP->date, &date);
     
-    ConnFormatClientAddr(sessionP->conn, &peerInfo);
+    ConnFormatClientAddr(sessionP->connP, &peerInfo);
     
-    xmlrpc_asprintf(&logline, "%s - %s - [%s] \"%s\" %d %d",
+    xmlrpc_asprintf(&logline, "%s - %s - [%s] \"%s\" %d %u",
                     peerInfo,
                     user,
                     date, 
                     sessionP->validRequest ?
                         sessionP->requestInfo.requestline : "???",
                     sessionP->status,
-                    sessionP->conn->outbytes
+                    sessionP->connP->outbytes
         );
     xmlrpc_strfree(peerInfo);
     xmlrpc_strfree(date);
     
-    if (logline) {
-        LogWrite(sessionP->conn->server, logline);
+    LogWrite(sessionP->connP->server, logline);
+
+    xmlrpc_strfree(logline);
         
-        xmlrpc_strfree(logline);
-    }
     return true;
 }
 
@@ -152,7 +157,7 @@ SessionLog(TSession * const sessionP) {
 void *
 SessionGetDefaultHandlerCtx(TSession * const sessionP) {
 
-    struct _TServer * const srvP = sessionP->conn->server->srvP;
+    struct _TServer * const srvP = sessionP->connP->server->srvP;
 
     return srvP->defaultHandlerContext;
 }
