@@ -3,7 +3,7 @@
 #include <unistd.h>
 #include <string.h>
 #include <errno.h>
-#include <wait.h>
+#include <sys/wait.h>
 #include <signal.h>
 
 #include "xmlrpc_config.h"
@@ -290,43 +290,3 @@ ThreadForks(void) {
 
 
 
-/*********************************************************************
-** Mutex
-*********************************************************************/
-
-/* As two processes don't share memory, there is nothing to synchronize,
-   so locking is a no-op.
-*/
-
-bool
-MutexCreate(TMutex ** const mutexP ATTR_UNUSED) {
-    return TRUE;
-}
-
-
-
-bool
-MutexLock(TMutex * const mutexP ATTR_UNUSED) {
-    return TRUE;
-}
-
-
-
-bool
-MutexUnlock(TMutex * const mutexP ATTR_UNUSED) {
-    return TRUE;
-}
-
-
-
-bool
-MutexTryLock(TMutex * const mutexP ATTR_UNUSED) {
-    return TRUE;
-}
-
-
-
-void
-MutexDestroy(TMutex * const mutexP ATTR_UNUSED) {
-
-}
