@@ -52,28 +52,27 @@
    such subordinate libraries as OpenSSL and Winsock.
 -----------------------------------------------------------------------------*/
 
+#define _XOPEN_SOURCE 600  /* Make sure strdup() is in <string.h> */
+
 #include "xmlrpc_config.h"
 
+#include <stdarg.h>
 #include <string.h>
 #include <stdlib.h>
 #include <errno.h>
 #include <assert.h>
 #include <limits.h>
-#if !MSVCRT
+#if HAVE_SYS_SELECT_H
 #include <sys/select.h>
 #endif
 #include <signal.h>
-
-#ifdef WIN32
-#include "curllink.h"
-#endif
+#include <stdio.h>
 
 #include "bool.h"
 #include "girmath.h"
 #include "mallocvar.h"
 #include "linklist.h"
 #include "girstring.h"
-#include "pthreadx.h"
 
 #include "xmlrpc-c/util.h"
 #include "xmlrpc-c/string_int.h"
@@ -81,14 +80,16 @@
 #include "xmlrpc-c/client_int.h"
 #include "xmlrpc-c/transport.h"
 #include "xmlrpc-c/time_int.h"
+#include "xmlrpc-c/lock.h"
+#include "xmlrpc-c/lock_platform.h"
 
 #include <curl/curl.h>
+#ifdef NEED_CURL_TYPES_H
 #include <curl/types.h>
+#endif
 #include <curl/easy.h>
 #include <curl/multi.h>
 
-#include "lock.h"
-#include "lock_pthread.h"
 #include "curltransaction.h"
 #include "curlmulti.h"
 #include "curlversion.h"
@@ -108,6 +109,31 @@ typedef struct rpc rpc;
 
 
 
+static void
+tracev(const char * const fmt,
+       va_list            args) {
+
+    vfprintf(stderr, fmt, args);
+
+    fprintf(stderr, "\n");
+}
+
+
+
+static void
+trace(const char * const fmt, ...) {
+
+    if (xmlrpc_trace_transport) {
+        va_list args;
+
+        va_start(args, fmt);
+        tracev(fmt, args);
+        va_end(args);
+    }
+}
+
+
+
 static int
 timeDiffMillisec(xmlrpc_timespec const minuend,
                  xmlrpc_timespec const subtractor) {
@@ -197,11 +223,26 @@ struct xmlrpc_client_transport {
            
            This is constant (the handle, not the object).
         */
+    bool dontAdvertise;
+        /* Don't identify to the server the XML-RPC engine we are using.  If
+           false, include a User-Agent HTTP header in all requests that
+           identifies the Xmlrpc-c and Curl libraries.
+
+           See also 'userAgent'.
+
+           This is constant.
+        */
     const char * userAgent;
-        /* Prefix for the User-Agent HTTP header, reflecting facilities
-           outside of Xmlrpc-c.  The actual User-Agent header consists
-           of this prefix plus information about Xmlrpc-c.  NULL means
-           none.
+        /* Information to include in a User-Agent HTTP header, reflecting
+           facilities outside of Xmlrpc-c.  
+
+           Null means none.
+
+           The full User-Agent header value is this information (if
+           'userAgent' is non-null) followed by identification of Xmlrpc-c
+           and Curl (if 'dontAdvertise' is false).  If 'userAgent' is null
+           and 'dontAdvertise' is true, we put no User-Agent header at all
+           in the request.
 
            This is constant.
         */
@@ -212,6 +253,8 @@ struct xmlrpc_client_transport {
            the transport to give up on whatever it is doing and return ASAP.
 
            NULL means none -- transport never gives up.
+
+           This is constant.
         */
 };
 
@@ -234,6 +277,10 @@ struct rpc {
         /* Routine to call to complete the RPC after it is complete HTTP-wise.
            NULL if none.
         */
+    xmlrpc_transport_progress progress;
+        /* Routine to call periodically to report the progress of transporting
+           the call and response.  NULL if none.
+        */
     struct xmlrpc_call_info * callInfoP;
         /* User's identifier for this RPC */
 };
@@ -258,7 +305,7 @@ unlockSyncCurlSession(struct xmlrpc_client_transport * const transportP) {
 static void
 initWindowsStuff(xmlrpc_env * const envP ATTR_UNUSED) {
 
-#if defined (WIN32)
+#if MSVCRT
     /* This is CRITICAL so that cURL-Win32 works properly! */
     
     /* So this commenter says, but I wonder why.  libcurl should do the
@@ -294,7 +341,7 @@ initWindowsStuff(xmlrpc_env * const envP ATTR_UNUSED) {
 static void
 termWindowsStuff(void) {
 
-#if defined (WIN32)
+#if MSVCRT
     WSACleanup();
 #endif
 }
@@ -331,8 +378,6 @@ pselectTimeout(xmlrpc_timeoutType const timeoutType,
     unsigned int selectTimeoutMillisec;
     xmlrpc_timespec retval;
 
-    selectTimeoutMillisec = 0; /* quiet compiler warning */
-
     /* We assume there is work to do at least every 3 seconds, because
        the Curl multi manager often has retries and other scheduled work
        that doesn't involve file handles on which we can select().
@@ -377,7 +422,7 @@ processCurlMessages(xmlrpc_env * const envP,
                 curlTransaction * curlTransactionP;
 
                 curl_easy_getinfo(curlMsg.easy_handle, CURLINFO_PRIVATE,
-                                  &curlTransactionP);
+                                  (void *)&curlTransactionP);
 
                 curlTransaction_finish(envP,
                                        curlTransactionP, curlMsg.data.result);
@@ -401,25 +446,25 @@ waitForWork(xmlrpc_env *       const envP,
    Update the Curl multi manager's file descriptor sets to indicate what
    work we found for it to do.
 
-   Wait under signal mask *sigmaskP.  The point of this is that Caller
-   can make sure that arrival of a signal of a certain class
-   interrupts our wait, even if the signal arrives shortly before we
-   begin waiting.  Caller blocks that signal class, then checks
-   whether a signal of that class has already been received.  If not,
-   he calls us with *sigmaskP indicating that class NOT blocked.
-   Thus, if a signal of that class arrived any time after Caller
-   checked, we will return immediately or when the signal arrives,
-   whichever is sooner.  Note that we can provide this service only
-   because pselect() has the same atomic unblock/wait feature.
+   Wait under signal mask *sigmaskP.  The point of this is that Caller can
+   make sure that arrival of a signal of a certain class interrupts our wait,
+   even if the signal arrives shortly before we begin waiting.  Caller blocks
+   that signal class, then checks whether a signal of that class has already
+   been received.  If not, he calls us with *sigmaskP indicating that class
+   NOT blocked.  Thus, if a signal of that class arrived any time after Caller
+   checked, we will return immediately and if it arrives while we're waiting,
+   we will return then.  Note that we can provide this service only because
+   pselect() has the same atomic unblock/wait feature.
    
-   If sigmaskP is NULL, wait under whatever the current signal mask
-   is.
+   If sigmaskP is NULL, wait under whatever the current signal mask is.
 -----------------------------------------------------------------------------*/
     fd_set readFdSet;
     fd_set writeFdSet;
     fd_set exceptFdSet;
     int maxFd;
 
+    trace("Waiting for work");
+
     curlMulti_fdset(envP, curlMultiP,
                     &readFdSet, &writeFdSet, &exceptFdSet, &maxFd);
     if (!envP->fault_occurred) {
@@ -434,6 +479,10 @@ waitForWork(xmlrpc_env *       const envP,
 
             int rc;
 
+            trace("No work available; waiting for a Curl file descriptor to "
+                  "be ready or %u.%03u sec",
+                  pselectTimeoutArg.tv_sec, pselectTimeoutArg.tv_nsec/1000000);
+
             rc = xmlrpc_pselect(maxFd+1, &readFdSet, &writeFdSet, &exceptFdSet,
                                 &pselectTimeoutArg, sigmaskP);
             
@@ -449,6 +498,7 @@ waitForWork(xmlrpc_env *       const envP,
                                       readFdSet, writeFdSet, exceptFdSet);
             }
         }
+        trace("Wait is over");
     }
 }
 
@@ -470,7 +520,7 @@ waitForWorkInt(xmlrpc_env *       const envP,
    waitForWork() will suffice.
 -----------------------------------------------------------------------------*/
     sigset_t callerBlockSet;
-#ifdef WIN32
+#ifdef _WIN32
     waitForWork(envP, curlMultiP, timeoutType, deadline, &callerBlockSet);
 #else
     sigset_t allSignals;
@@ -483,6 +533,8 @@ waitForWorkInt(xmlrpc_env *       const envP,
     
     if (*interruptP == 0)
         waitForWork(envP, curlMultiP, timeoutType, deadline, &callerBlockSet);
+    else
+        trace("Not waiting because interrupt flag is set\n");
 
     sigprocmask(SIG_SETMASK, &callerBlockSet, NULL);
 #endif
@@ -508,18 +560,23 @@ doCurlWork(xmlrpc_env * const envP,
    again.
 -----------------------------------------------------------------------------*/
     bool immediateWorkToDo;
-    int runningHandles;
+    int runningHandleCt;
+
+    trace("Calling libcurl to perform all immediate work");
 
     immediateWorkToDo = true;  /* initial assumption */
 
     while (immediateWorkToDo && !envP->fault_occurred) {
         curlMulti_perform(envP, curlMultiP,
-                          &immediateWorkToDo, &runningHandles);
+                          &immediateWorkToDo, &runningHandleCt);
     }
 
     /* We either did all the work that's ready to do or hit an error. */
 
     if (!envP->fault_occurred) {
+        trace("libcurl has performed all immediate work; %d tasks "
+              "(file handles) still running", runningHandleCt);
+
         /* The work we did may have resulted in asynchronous messages
            (asynchronous to the thing they refer to, not to us, of course).
            In particular the message "Curl transaction has completed".
@@ -527,7 +584,7 @@ doCurlWork(xmlrpc_env * const envP,
         */
         processCurlMessages(envP, curlMultiP);
 
-        *transStillRunningP = runningHandles > 0;
+        *transStillRunningP = runningHandleCt > 0;
     }
 }
 
@@ -547,23 +604,41 @@ finishCurlMulti(xmlrpc_env *       const envP,
 
    Don't return until all the Curl transactions are done or we time out.
 
+   if 'interruptP' is non-null, it points to a value which is normally zero,
+   but is nonzero when Caller wants us to abort all the transactions and
+   return ASAP.
+
    The *interruptP flag alone will not interrupt us.  We will wait in
    spite of it for all Curl transactions to complete.  *interruptP
    just gives us a hint that the Curl transactions are being
    interrupted, so we know there is work to do for them.  (The way it
    works is Caller sets up a "progress" function that checks the same
    interrupt flag and reports "kill me."  When we see the interrupt
-   flag, we call that progress function and get the message).
+   flag, we tell libcurl to do whatever work there is to do, and as part of
+   that, libcurl calls the progress function, gets the "kill me" message,
+   and passes that on to us).
 -----------------------------------------------------------------------------*/
+    /* For integrity, we make sure we don't let *interruptP interrupt our
+       wait for work more than once.  That way, if for any reason libcurl
+       fails to call the progress function, or the progress function fails
+       to notice the interrupt flag and tell libcurl to abort, or libcurl
+       does abort as told, we don't have a busy loop of calls to
+       doCurlWork().
+
+       'curlCalledSinceInterrupt' is part of this logic.
+    */
+
     bool rpcStillRunning;
     bool timedOut;
+    bool curlCalledSinceInterrupt;
 
     rpcStillRunning = true;  /* initial assumption */
     timedOut = false;
+    curlCalledSinceInterrupt = false;
     
     while (rpcStillRunning && !timedOut && !envP->fault_occurred) {
 
-        if (interruptP) {
+        if (interruptP && !curlCalledSinceInterrupt) {
             waitForWorkInt(envP, curlMultiP, timeoutType, deadline,
                            interruptP);
         } else 
@@ -572,12 +647,16 @@ finishCurlMulti(xmlrpc_env *       const envP,
         if (!envP->fault_occurred) {
             xmlrpc_timespec nowTime;
 
-            /* doCurlWork() (among other things) finds Curl
-               transactions that user wants to abort and finishes
-               them.
+            /* doCurlWork() (among other things) finds Curl transactions that
+               user wants to abort and finishes them.  (This is by virtue
+               of libcurl calling its progress function when we tell it to do
+               all available work).
             */
+            if (interruptP && *interruptP)
+                curlCalledSinceInterrupt = true;
+
             doCurlWork(envP, curlMultiP, &rpcStillRunning);
-            
+
             xmlrpc_gettimeofday(&nowTime);
             
             timedOut = (timeoutType == timeout_yes &&
@@ -599,7 +678,8 @@ getTimeoutParm(xmlrpc_env *                          const envP,
     else {
         if (curlHasNosignal()) {
             /* libcurl takes a 'long' in milliseconds for the timeout value */
-            if ((curlXportParmsP->timeout + 999) / 1000 > LONG_MAX)
+            if ((unsigned)(long)(curlXportParmsP->timeout) !=
+                curlXportParmsP->timeout)
                 xmlrpc_faultf(envP, "Timeout value %u is too large.",
                               curlXportParmsP->timeout);
             else
@@ -614,6 +694,19 @@ getTimeoutParm(xmlrpc_env *                          const envP,
 
 
 
+static void
+setVerbose(bool * const verboseP) {
+
+    const char * const xmlrpcTraceCurl = getenv("XMLRPC_TRACE_CURL");
+
+    if (xmlrpcTraceCurl)
+        *verboseP = true;
+    else
+        *verboseP = false;
+}
+
+
+
 static void
 getXportParms(xmlrpc_env *                          const envP,
               const struct xmlrpc_curl_xportparms * const curlXportParmsP,
@@ -652,6 +745,11 @@ getXportParms(xmlrpc_env *                          const envP,
     else
         transportP->userAgent = strdup(curlXportParmsP->user_agent);
     
+    if (!curlXportParmsP || parmSize < XMLRPC_CXPSIZE(dont_advertise))
+        transportP->dontAdvertise = false;
+    else
+        transportP->dontAdvertise = curlXportParmsP->dont_advertise;
+    
     if (!curlXportParmsP || parmSize < XMLRPC_CXPSIZE(network_interface))
         curlSetupP->networkInterface = NULL;
     else if (curlXportParmsP->network_interface == NULL)
@@ -706,7 +804,7 @@ getXportParms(xmlrpc_env *                          const envP,
     else
         curlSetupP->sslKeyType = strdup(curlXportParmsP->sslkeytype);
     
-        if (!curlXportParmsP || parmSize < XMLRPC_CXPSIZE(sslkeypasswd))
+    if (!curlXportParmsP || parmSize < XMLRPC_CXPSIZE(sslkeypasswd))
         curlSetupP->sslKeyPasswd = NULL;
     else if (curlXportParmsP->sslkeypasswd == NULL)
         curlSetupP->sslKeyPasswd = NULL;
@@ -765,6 +863,47 @@ getXportParms(xmlrpc_env *                          const envP,
     else
         curlSetupP->sslCipherList = strdup(curlXportParmsP->ssl_cipher_list);
 
+    if (!curlXportParmsP || parmSize < XMLRPC_CXPSIZE(proxy))
+        curlSetupP->proxy = NULL;
+    else if (curlXportParmsP->proxy == NULL)
+        curlSetupP->proxy = NULL;
+    else
+        curlSetupP->proxy = strdup(curlXportParmsP->proxy);
+
+    if (!curlXportParmsP || parmSize < XMLRPC_CXPSIZE(proxy_port))
+        curlSetupP->proxyPort = 8080;
+    else
+        curlSetupP->proxyPort = curlXportParmsP->proxy_port;
+
+    if (!curlXportParmsP || parmSize < XMLRPC_CXPSIZE(proxy_auth))
+        curlSetupP->proxyAuth = CURLAUTH_BASIC;
+    else
+        curlSetupP->proxyAuth = curlXportParmsP->proxy_auth;
+
+    if (!curlXportParmsP || parmSize < XMLRPC_CXPSIZE(proxy_userpwd))
+        curlSetupP->proxyUserPwd = NULL;
+    else if (curlXportParmsP->proxy_userpwd == NULL)
+        curlSetupP->proxyUserPwd = NULL;
+    else
+        curlSetupP->proxyUserPwd = strdup(curlXportParmsP->proxy_userpwd);
+
+    if (!curlXportParmsP || parmSize < XMLRPC_CXPSIZE(proxy_type))
+        curlSetupP->proxyType = CURLPROXY_HTTP;
+    else
+        curlSetupP->proxyType = curlXportParmsP->proxy_type;
+
+    if (!curlXportParmsP || parmSize < XMLRPC_CXPSIZE(gssapi_delegation))
+        curlSetupP->gssapiDelegation = false;
+    else
+        curlSetupP->gssapiDelegation = !!curlXportParmsP->gssapi_delegation;
+
+    if (!curlXportParmsP || parmSize < XMLRPC_CXPSIZE(referer))
+        curlSetupP->referer = NULL;
+    else if (curlXportParmsP->referer == NULL)
+        curlSetupP->referer = NULL;
+    else
+        curlSetupP->referer = strdup(curlXportParmsP->referer);
+
     getTimeoutParm(envP, curlXportParmsP, parmSize, &curlSetupP->timeout);
 }
 
@@ -803,6 +942,12 @@ freeXportParms(const struct xmlrpc_client_transport * const transportP) {
         xmlrpc_strfree(curlSetupP->networkInterface);
     if (transportP->userAgent)
         xmlrpc_strfree(transportP->userAgent);
+    if (curlSetupP->proxy)
+        xmlrpc_strfree(curlSetupP->proxy);
+    if (curlSetupP->proxyUserPwd)
+        xmlrpc_strfree(curlSetupP->proxyUserPwd);
+    if (curlSetupP->referer)
+        xmlrpc_strfree(curlSetupP->referer);
 }
 
 
@@ -863,7 +1008,7 @@ static void
 makeSyncCurlSession(xmlrpc_env *                     const envP,
                     struct xmlrpc_client_transport * const transportP) {
 
-    transportP->syncCurlSessionLockP = curlLock_create_pthread();
+    transportP->syncCurlSessionLockP = xmlrpc_lock_create();
     if (transportP->syncCurlSessionLockP == NULL)
         xmlrpc_faultf(envP, "Unable to create lock for "
                       "synchronous Curl session.");
@@ -922,6 +1067,8 @@ create(xmlrpc_env *                      const envP,
     if (transportP == NULL)
         xmlrpc_faultf(envP, "Unable to allocate transport descriptor.");
     else {
+        setVerbose(&transportP->curlSetupStuff.verbose);
+
         transportP->interruptP = NULL;
 
         transportP->asyncCurlMultiP = curlMulti_create();
@@ -1042,7 +1189,7 @@ performCurlTransaction(xmlrpc_env *      const envP,
 
         if (!envP->fault_occurred) {
             /* Curl session completed OK.  But did HTTP transaction
-               work?
+               it prosecuted work?
             */
             curlTransaction_getError(curlTransactionP, envP);
         }
@@ -1083,6 +1230,7 @@ createRpc(xmlrpc_env *                     const envP,
           xmlrpc_mem_block *               const callXmlP,
           xmlrpc_mem_block *               const responseXmlP,
           xmlrpc_transport_asynch_complete       complete, 
+          xmlrpc_transport_progress              progress,
           struct xmlrpc_call_info *        const callInfoP,
           rpc **                           const rpcPP) {
 
@@ -1092,21 +1240,33 @@ createRpc(xmlrpc_env *                     const envP,
     if (rpcP == NULL)
         xmlrpc_faultf(envP, "Couldn't allocate memory for rpc object");
     else {
+        curlt_progressFn * curlProgressFn;
+
+        if (progress || clientTransportP->interruptP)
+            curlProgressFn = &curlTransactionProgress;
+        else {
+            /* There's nothing for curlTransactionProgress() to do, so save
+               the time and complexity of calling it.
+            */
+            curlProgressFn = NULL;
+        }
         rpcP->transportP   = clientTransportP;
         rpcP->curlSessionP = curlSessionP;
         rpcP->callInfoP    = callInfoP;
         rpcP->complete     = complete;
+        rpcP->progress     = progress;
         rpcP->responseXmlP = responseXmlP;
 
         curlTransaction_create(envP,
                                curlSessionP,
                                serverP,
                                callXmlP, responseXmlP, 
+                               clientTransportP->dontAdvertise,
                                clientTransportP->userAgent,
                                &clientTransportP->curlSetupStuff,
                                rpcP,
                                complete ? &finishRpcCurlTransaction : NULL,
-                               &curlTransactionProgress,
+                               curlProgressFn,
                                &rpcP->curlTransactionP);
         if (!envP->fault_occurred) {
             if (envP->fault_occurred)
@@ -1190,15 +1350,51 @@ static curlt_progressFn curlTransactionProgress;
 
 static void
 curlTransactionProgress(void * const context,
+                        double const dlTotal,
+                        double const dlNow,
+                        double const ulTotal,
+                        double const ulNow,
                         bool * const abortP) {
+/*----------------------------------------------------------------------------
+   This is equivalent to a Curl "progress function" (the curlTransaction
+   object just passes through the call from libcurl).
 
+   The curlTransaction calls this once a second telling us how much
+   data has transferred.  If the transport user has set up a progress
+   function, we call that with this progress information.  That 
+   function might e.g. display a progress bar.
+
+   Additionally, the curlTransaction gives us the opportunity to tell it
+   to abort the transaction, which we do if the user has set his
+   "interrupt" flag (which he registered with the transport when he
+   created it).
+-----------------------------------------------------------------------------*/
     rpc * const rpcP = context;
+    struct xmlrpc_client_transport * const transportP = rpcP->transportP;
 
     assert(rpcP);
+    assert(transportP);
 
-    if (rpcP->transportP->interruptP)
-        *abortP = *rpcP->transportP->interruptP;
-    else
+    trace("Progress function called back by libcurl");
+
+    if (rpcP->progress) {
+        struct xmlrpc_progress_data progressData;
+
+        trace("Calling transport client's progress function with %u %u %u %u",
+              dlTotal, dlNow, ulTotal, ulNow);
+
+        progressData.response.total = dlTotal;
+        progressData.response.now   = dlNow;
+        progressData.call.total     = ulTotal;
+        progressData.call.now       = ulNow;
+
+        rpcP->progress(rpcP->callInfoP, progressData);
+    }
+    if (transportP->interruptP) {
+        trace("Interrupt flag is set; "
+              "directing libcurl to abort the transaction");
+        *abortP = *transportP->interruptP;
+    } else
         *abortP = false;
 }
 
@@ -1210,6 +1406,7 @@ sendRequest(xmlrpc_env *                     const envP,
             const xmlrpc_server_info *       const serverP,
             xmlrpc_mem_block *               const callXmlP,
             xmlrpc_transport_asynch_complete       complete,
+            xmlrpc_transport_progress              progress,
             struct xmlrpc_call_info *        const callInfoP) {
 /*----------------------------------------------------------------------------
    Initiate an XML-RPC rpc asynchronously.  Don't wait for it to go to
@@ -1232,7 +1429,7 @@ sendRequest(xmlrpc_env *                     const envP,
                           "curl_easy_init() failed.");
         else {
             createRpc(envP, clientTransportP, curlSessionP, serverP,
-                      callXmlP, responseXmlP, complete, callInfoP,
+                      callXmlP, responseXmlP, complete, progress, callInfoP,
                       &rpcP);
             
             if (!envP->fault_occurred) {
@@ -1355,7 +1552,7 @@ call(xmlrpc_env *                     const envP,
         createRpc(envP, clientTransportP, clientTransportP->syncCurlSessionP,
                   serverP,
                   callXmlP, responseXmlP,
-                  NULL, NULL,
+                  NULL, NULL, NULL,
                   &rpcP);
 
         if (!envP->fault_occurred) {
@@ -1397,7 +1594,7 @@ setupGlobalConstants(xmlrpc_env * const envP) {
 static void
 teardownGlobalConstants(void) {
 /*----------------------------------------------------------------------------
-   See longwinded discussionof the global constant issue at the top of
+   See longwinded discussion of the global constant issue at the top of
    this file.
 -----------------------------------------------------------------------------*/
     curl_global_cleanup();
