@@ -165,37 +165,33 @@ struct decompTreeNode {
 
 /* prototype for recursive calls */
 static void
-releaseDecomposition(const struct decompTreeNode * const decompRootP,
-                     bool                          const oldstyleMemMgmt);
+releaseDecomposition(const struct decompTreeNode * const decompRootP);
 
 
 static void
-releaseDecompArray(struct arrayDecomp const arrayDecomp,
-                   bool               const oldstyleMemMgmt) {
+releaseDecompArray(struct arrayDecomp const arrayDecomp) {
 
     unsigned int i;
     for (i = 0; i < arrayDecomp.itemCnt; ++i) {
-        releaseDecomposition(arrayDecomp.itemArray[i], oldstyleMemMgmt);
+        releaseDecomposition(arrayDecomp.itemArray[i]);
     }
 }
 
 
+
 static void
-releaseDecompStruct(struct structDecomp const structDecomp,
-                    bool                const oldstyleMemMgmt) {
+releaseDecompStruct(struct structDecomp const structDecomp) {
 
     unsigned int i;
     for (i = 0; i < structDecomp.mbrCnt; ++i) {
-        releaseDecomposition(structDecomp.mbrArray[i].decompTreeP,
-                             oldstyleMemMgmt);
+        releaseDecomposition(structDecomp.mbrArray[i].decompTreeP);
     }
 }
 
 
 
 static void
-releaseDecomposition(const struct decompTreeNode * const decompRootP,
-                     bool                          const oldstyleMemMgmt) {
+releaseDecomposition(const struct decompTreeNode * const decompRootP) {
 /*----------------------------------------------------------------------------
    Assuming that Caller has decomposed something according to 'decompRootP',
    release whatever resources the decomposed information occupies.
@@ -236,10 +232,10 @@ releaseDecomposition(const struct decompTreeNode * const decompRootP,
         xmlrpc_DECREF(*decompRootP->store.TstructVal.valueP);
         break;
     case '(':
-        releaseDecompArray(decompRootP->store.Tarray, oldstyleMemMgmt);
+        releaseDecompArray(decompRootP->store.Tarray);
         break;
     case '{':
-        releaseDecompStruct(decompRootP->store.Tstruct, oldstyleMemMgmt);
+        releaseDecompStruct(decompRootP->store.Tstruct);
         break;
     }
 }
@@ -310,11 +306,12 @@ parsearray(xmlrpc_env *         const envP,
             }
         }
         if (envP->fault_occurred) {
-            /* Release the items we completed before we failed. */
-            unsigned int i;
-            for (i = 0; i < doneCnt; ++i)
-                releaseDecomposition(arrayDecomp.itemArray[i],
-                                     oldstyleMemMgmt);
+            if (!oldstyleMemMgmt) {
+                /* Release the items we completed before we failed. */
+                unsigned int i;
+                for (i = 0; i < doneCnt; ++i)
+                    releaseDecomposition(arrayDecomp.itemArray[i]);
+            }
         }
     }
 }
@@ -351,10 +348,12 @@ parsestruct(xmlrpc_env *        const envP,
     }
 
     if (envP->fault_occurred) {
-        unsigned int i;
-        for (i = 0; i < doneCount; ++i)
-            releaseDecomposition(structDecomp.mbrArray[i].decompTreeP,
-                                 oldstyleMemMgmt);
+        if (!oldstyleMemMgmt) {
+            /* Release the items we completed before we failed. */
+            unsigned int i;
+            for (i = 0; i < doneCount; ++i)
+                releaseDecomposition(structDecomp.mbrArray[i].decompTreeP);
+        }
     }
 }
 
@@ -712,7 +711,7 @@ buildArrayDecompBranch(xmlrpc_env *            const envP,
         if (itemCnt >= ARRAY_SIZE(decompNodeP->store.Tarray.itemArray))
             xmlrpc_faultf(envP, "Too many array items in format string.  "
                           "The most items you can have for an array in "
-                          "a format string is %u.",
+                          "a format string is %u.", (unsigned)
                           ARRAY_SIZE(decompNodeP->store.Tarray.itemArray));
         else {
             struct decompTreeNode * itemNodeP;
@@ -850,7 +849,7 @@ buildStructDecompBranch(xmlrpc_env *            const envP,
             xmlrpc_faultf(envP,
                           "Too many structure members in format string.  "
                           "The most members you can specify in "
-                          "a format string is %u.",
+                          "a format string is %u.", (unsigned)
                           ARRAY_SIZE(decompNodeP->store.Tstruct.mbrArray));
         else {
             struct mbrDecomp * const mbrP =
@@ -1059,9 +1058,8 @@ createDecompTree(xmlrpc_env *             const envP,
 
         if (envP->fault_occurred)
             destroyDecompTree(decompRootP);
-        else
-            *decompRootPP = decompRootP;
     }
+    *decompRootPP = decompRootP;
 }
 
 
