@@ -4,9 +4,9 @@
 
    The printf format specifiers we use appear to be entirely standard,
    except for the "long long" one, which is %I64 on Windows and %lld
-   everywhere else.  So for that, we use the C99 standard macro PRId64,
-   which is defined by inttypes.h.  Ironically, Windows doesn't have
-   inttypes.h either, but we have int.h instead.
+   everywhere else.  We could use the C99 standard macro PRId64 for that,
+   but on at least one 64-bit-long GNU compiler, PRId64 is "ld", which is
+   considered to be incompatible with long long.  So we have XMLRPC_PRId64.
 */
 
 #include "xmlrpc_config.h"
@@ -277,6 +277,74 @@ xmlrpc_serialize_base64_data(xmlrpc_env *       const envP,
 
 
 
+static void
+serializeDatetime(xmlrpc_env *       const envP,
+                  xmlrpc_mem_block * const outputP,
+                  xmlrpc_value *     const valueP) {
+/*----------------------------------------------------------------------------
+   Add to *outputP the content of a <value> element to represent
+   the datetime value *valueP.  I.e.
+   "<dateTime.iso8601> ... </dateTime.iso8601>".
+-----------------------------------------------------------------------------*/
+
+    addString(envP, outputP, "<dateTime.iso8601>");
+    if (!envP->fault_occurred) {
+        char dtString[64];
+
+        snprintf(dtString, sizeof(dtString),
+                 "%u%02u%02uT%02u:%02u:%02u",
+                 valueP->_value.dt.Y,
+                 valueP->_value.dt.M,
+                 valueP->_value.dt.D,
+                 valueP->_value.dt.h,
+                 valueP->_value.dt.m,
+                 valueP->_value.dt.s);
+
+        if (valueP->_value.dt.u != 0) {
+            char usecString[64];
+            assert(valueP->_value.dt.u < 1000000);
+            snprintf(usecString, sizeof(usecString), ".%06u",
+                     valueP->_value.dt.u);
+            STRSCAT(dtString, usecString);
+        }
+        addString(envP, outputP, dtString);
+
+        if (!envP->fault_occurred) {
+            addString(envP, outputP, "</dateTime.iso8601>");
+        }
+    }
+}
+
+
+
+static void
+serializeStructMember(xmlrpc_env *       const envP,
+                      xmlrpc_mem_block * const outputP,
+                      xmlrpc_value *     const memberKeyP,
+                      xmlrpc_value *     const memberValueP,
+                      xmlrpc_dialect     const dialect) {
+    
+    addString(envP, outputP, "<member><name>");
+
+    if (!envP->fault_occurred) {
+        serializeUtf8MemBlock(envP, outputP, &memberKeyP->_block);
+
+        if (!envP->fault_occurred) {
+            addString(envP, outputP, "</name>"CRLF);
+
+            if (!envP->fault_occurred) {
+                xmlrpc_serialize_value2(envP, outputP, memberValueP, dialect);
+
+                if (!envP->fault_occurred) {
+                    addString(envP, outputP, "</member>"CRLF);
+                }
+            }
+        }
+    }
+}
+
+
+
 static void 
 serializeStruct(xmlrpc_env *       const envP,
                 xmlrpc_mem_block * const outputP,
@@ -286,37 +354,26 @@ serializeStruct(xmlrpc_env *       const envP,
    Add to *outputP the content of a <value> element to represent
    the structure value *valueP.  I.e. "<struct> ... </struct>".
 -----------------------------------------------------------------------------*/
-    size_t size;
-    size_t i;
-    xmlrpc_value * memberKeyP;
-    xmlrpc_value * memberValueP;
-
     addString(envP, outputP, "<struct>"CRLF);
-    XMLRPC_FAIL_IF_FAULT(envP);
+    if (!envP->fault_occurred) {
+        unsigned int const size = xmlrpc_struct_size(envP, structP);
+        if (!envP->fault_occurred) {
+            unsigned int i;
+            for (i = 0; i < size && !envP->fault_occurred; ++i) {
+                xmlrpc_value * memberKeyP;
+                xmlrpc_value * memberValueP;
 
-    size = xmlrpc_struct_size(envP, structP);
-    XMLRPC_FAIL_IF_FAULT(envP);
-    for (i = 0; i < size; ++i) {
-        xmlrpc_struct_get_key_and_value(envP, structP, i,
-                                        &memberKeyP, &memberValueP);
-        XMLRPC_FAIL_IF_FAULT(envP);
-        addString(envP, outputP, "<member><name>");
-        XMLRPC_FAIL_IF_FAULT(envP);
-        serializeUtf8MemBlock(envP, outputP, &memberKeyP->_block);
-        XMLRPC_FAIL_IF_FAULT(envP);
-        addString(envP, outputP, "</name>"CRLF);
-        XMLRPC_FAIL_IF_FAULT(envP);
-        xmlrpc_serialize_value2(envP, outputP, memberValueP, dialect);
-        XMLRPC_FAIL_IF_FAULT(envP);
-        addString(envP, outputP, "</member>"CRLF);
-        XMLRPC_FAIL_IF_FAULT(envP);
+                xmlrpc_struct_get_key_and_value(envP, structP, i,
+                                                &memberKeyP, &memberValueP);
+                if (!envP->fault_occurred) {
+                    serializeStructMember(envP, outputP,
+                                          memberKeyP, memberValueP, dialect);
+                }
+            }
+            if (!envP->fault_occurred)
+                addString(envP, outputP, "</struct>");
+        }
     }
-
-    addString(envP, outputP, "</struct>");
-    XMLRPC_FAIL_IF_FAULT(envP);
-
-cleanup:
-    return;
 }
 
 
@@ -397,13 +454,7 @@ formatValueContent(xmlrpc_env *       const envP,
     } break;
 
     case XMLRPC_TYPE_DATETIME:
-        addString(envP, outputP, "<dateTime.iso8601>");
-        if (!envP->fault_occurred) {
-            serializeUtf8MemBlock(envP, outputP, &valueP->_block);
-            if (!envP->fault_occurred) {
-                addString(envP, outputP, "</dateTime.iso8601>");
-            }
-        }
+        serializeDatetime(envP, outputP, valueP);
         break;
 
     case XMLRPC_TYPE_STRING:
@@ -508,9 +559,9 @@ xmlrpc_serialize_params2(xmlrpc_env *       const envP,
     addString(envP, outputP, "<params>"CRLF);
     if (!envP->fault_occurred) {
         /* Serialize each parameter. */
-        size_t const paramCount = xmlrpc_array_size(envP, paramArrayP);
+        int const paramCount = xmlrpc_array_size(envP, paramArrayP);
         if (!envP->fault_occurred) {
-            size_t paramSeq;
+            int paramSeq;
             for (paramSeq = 0;
                  paramSeq < paramCount && !envP->fault_occurred;
                  ++paramSeq) {
